#!/usr/bin/php -q
<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2006 Reinhard Scheck aka gandalf                          |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 */


function pollperf_fill_table() {
	global $config, $debug;
	if ($debug) { print("PollPerf Fill Table\n");}

	/* suppress warnings */
	error_reporting(0);

	/* install the boost error handler */
	set_error_handler("pollperf_error_handler");

	/*
	 * some constants and search patterns
	 */
	$logfile 				= read_config_option("path_cactilog");
	$i 						= 0;
	$poller_runs 			= 0;	# number of poller runs, count "SYSTEM STATS"
	$stats_found			= 0;	# number of poller runs, that include runtime statistics
	/* search string			for Host entries */
	$search_Host 			= '/^(\d*\/\d*\/\d* \d*:\d*:\d* [A|P]M).+Host\[(\d*)\](.*)/';
	/* search string			for DS entries */
	$search_DS 				= '/^.+DS\[\d*\]/';
	/* search string			for Errors or Warnings */
	$search_ErrorWarning 	= '/^.*ERROR|WARNING/';
	/* search string			for SYSTEM STATS */
	$search_System_Stats 	= '/^.*SYSTEM STATS/';

	/* fetch the hosts table
	 *  */
	$total_rows = db_fetch_cell("select
		COUNT(host.id)
		FROM host"
		);
	if ($debug) {print("PollPerf #Hosts: $total_rows\n");}

	/* get $hosts() from MySql */
	$hosts = db_fetch_assoc("SELECT
		host.id
		FROM host"
		);
	if ($debug) {print_r($hosts);}

	/*
	 * delete and re-create table
	 * truncating would be faster, but this would require some logic to
	 * detect potential changes in table structure for later releases
	 */
	pollperf_delete_table();
	pollperf_create_table();

	/* open log/cacti.log for reading, hope we get it while cacti is NOT writing to it */
	/* to get $loghosts() array                                                        */
	$fp = fopen($logfile, "r");

	while ($line = fgets($fp, 4096)) {
		/*
		 * search for Host entries
		 * 03/04/2006 09:55:03 PM - CACTID: Poller[0] Host[1] SNMP Result: Host responded to SNMP
		 * and remember time, host_id and trailing text
    	 */

    	
    	/* filter WARNING and ERROR messages, they may contain Host[..] entries so better ignore them */
        if ( preg_match($search_ErrorWarning, $line, $match) ) {continue; }

        
        /* get Host entries */
        if ( preg_match($search_Host, $line, $match) ) {
        	$time 		= $match[1];
        	$host_id 	= $match[2];
        	$trail 		= $match[3];

			/* store all log data in loghosts array */
        	if (!isset($loghosts[$host_id]["start"])) {
        		/* this is the first entry for this host_id? */
        		$loghosts[$host_id]["start"] = $time;
        		$loghosts[$host_id]["ds_count"] = 0;
        	}
        	/* remember each entry for a host as its last one */
        	$loghosts[$host_id]["end"] = $time;
			/* count if DS entry found */
	        if ( preg_match($search_DS, $trail, $match) ) { $loghosts[$host_id]["ds_count"]++; }
        }

    	
    	/* if SYSTEM STATS, where at the end of a poller's run */
        if ( preg_match($search_System_Stats, $line, $match) ) {
        	$poller_runs++;
			if (isset($loghosts)) {$stats_found++;}
        	
			/* merge $hosts() and $loghosts() arrays here */
			for ($i = 0; $i < sizeof($hosts); $i++) {
				/* found sth for this host in the logfile? $hosts[$i]["id"] is id of current host*/
				$current_id = $hosts[$i]["id"];
				$found = isset($loghosts[$current_id]["end"]);
				if ($found) {
					$hosts[$i]["lstart"] 	= $loghosts[$current_id]["start"];
					$hosts[$i]["lend"] 		= $loghosts[$current_id]["end"];
					if (isset($hosts[$i]["ds_count"])) {
						$hosts[$i]["ds_count"] += $loghosts[$current_id]["ds_count"];
					} else {
						$hosts[$i]["ds_count"] = $loghosts[$current_id]["ds_count"];
					}
					if (isset($hosts[$i]["diff"])) {
						/* add 0.5 sec due to time resolution */
						$hosts[$i]["diff"]	+= strtotime($loghosts[$current_id]["end"]) -  strtotime($loghosts[$current_id]["start"]) + 0.5;
					} else {
						$hosts[$i]["diff"]	= strtotime($loghosts[$current_id]["end"]) -  strtotime($loghosts[$current_id]["start"]) + 0.5;
					}
					if ($hosts[$i]["ds_count"] > 0) {
						$hosts[$i]["rate"]	= $hosts[$i]["diff"]/$hosts[$i]["ds_count"];
					} else {
						$hosts[$i]["rate"]	= 0;
					}
				}
			}
			/* destroy loghosts array to start from scratch for new poller logrecords */
			unset($loghosts);
		}

    }
    fclose($fp);

	
	/*
	 * compute some more statistics now, that all records are scanned
	 * some weird php logic taken from php.net
	 */
	if ($debug) {print_r($hosts);}
	reset($hosts);
	while (list($key, $value) = each($hosts)) {
		$host =& $hosts[$key];
		$host["ds_per_run"] 	= isset($host["ds_count"]) 	? round($host["ds_count"]/$stats_found, 2)	: "None";
		$host["dur_per_run"] 	= isset($host["diff"]) 		? round($host["diff"]/$stats_found, 2)		: "None";
		$host["rate"]		 	= isset($host["rate"]) 		? round($host["rate"], 2)					: "None";
		# re-format datetime
		$host["lstart"]			= date('Y-m-d H:i:s', strtotime($host["lstart"]));
		$host["lend"]			= date('Y-m-d H:i:s', strtotime($host["lend"]));

		$sql = "REPLACE INTO plugin_pollperf_data (	`id`, `lstart`,	`lend`,	`ds_count`, `diff`, `rate`, `ds_per_run`, `dur_per_run`) VALUES ('"
			. sql_sanitize($host['id']) . "', '"
			. sql_sanitize($host['lstart']) . "', '"
			. sql_sanitize($host['lend']) . "', '"
			. sql_sanitize($host['ds_count']) . "', '"
			. sql_sanitize($host['diff']) . "', '"
			. sql_sanitize($host['rate']) . "', '"
			. sql_sanitize($host['ds_per_run']) . "', '"
			. sql_sanitize($host['dur_per_run']) . "'"
			. ')';
		if ($debug) {print($sql . "\n");}
		db_execute($sql);
		if ($debug) {print_r($host);}
	}

	# some log file statistics
	if ($debug) {print("Stats Found: $stats_found\n");}
	db_execute("REPLACE INTO settings (name, value) VALUES ('plugin_pollperf_stats_found', '$stats_found')");
	if ($debug) {print("Total Runs: $poller_runs\n");}
	db_execute("REPLACE INTO settings (name, value) VALUES ('plugin_pollperf_total_runs', '$poller_runs')");

	/* restore original error handler */
	restore_error_handler();

}

function pollperf_delete_table () {
	$sql = "DROP TABLE IF EXISTS `plugin_pollperf_data`";
	db_execute($sql);
}

function pollperf_create_table () {
	$sql = "CREATE TABLE IF NOT EXISTS `plugin_pollperf_data` (
	`id` 		mediumint(8) unsigned NOT NULL COMMENT 'Index',
	`lstart` 	datetime NOT NULL default '0000-00-00 00:00:00',
	`lend` 		datetime NOT NULL default '0000-00-00 00:00:00',
	`ds_count`	mediumint(8) unsigned NOT NULL default '0',
	`diff`		mediumint(8) unsigned NOT NULL default '0',
	`rate`		decimal(6,2) default '9.99',
	`ds_per_run`	mediumint(8) unsigned NOT NULL default '0',
	`dur_per_run`	decimal(6,2) default '9.99',
	PRIMARY KEY  (`id`)
	) ENGINE=MyISAM COMMENT='Poller Performance Data read from cacti log file'";
	db_execute($sql);
}

/*	display_help - displays the usage of the function */
function display_help () {
	print "Poller Performance Log File Parser Version 0.3, Copyright 2007 - Reinhard Scheck\n\n";
	print "usage: pollperf_parse.php [-d] [-h] [--help] [-v] [-V] [--version]\n\n";
#	print "-f               - Force the execution of a parsing process\n";
	print "-d               - Display verbose output during execution\n";
	print "-v -V --version  - Display this help message\n";
	print "-h --help        - display this help message\n";
}

function sig_handler($signo) {
	switch ($signo) {
		case SIGTERM:
		case SIGINT:
		case SIGHUP:
			cacti_log("WARNING: PollPerf_Parse terminated by user", FALSE, "POLLPERF");

			exit;
			break;
		default:
			/* ignore all other signals */
	}

}

/**
 * PHP error handler
 * @arg $errno
 * @arg $errmsg
 * @arg $filename
 * @arg $linenum
 * @arg $vars
 */
function pollperf_error_handler($errno, $errmsg, $filename, $linenum, $vars) {
	global $debug;
	if ($debug) {
		/* define all error types */
		$errortype = array(
			E_ERROR             => 'Error',
			E_WARNING           => 'Warning',
			E_PARSE             => 'Parsing Error',
			E_NOTICE            => 'Notice',
			E_CORE_ERROR        => 'Core Error',
			E_CORE_WARNING      => 'Core Warning',
			E_COMPILE_ERROR     => 'Compile Error',
			E_COMPILE_WARNING   => 'Compile Warning',
			E_USER_ERROR        => 'User Error',
			E_USER_WARNING      => 'User Warning',
			E_USER_NOTICE       => 'User Notice',
#			E_STRICT            => 'Runtime Notice',
#			E_RECOVERABLE_ERROR => 'Catchable Fatal Error'
		);

		/* create an error string for the log */
		$err = "ERRNO:'"  . $errno   . "' TYPE:'"    . $errortype[$errno] .
			"' MESSAGE:'" . $errmsg  . "' IN FILE:'" . $filename .
			"' LINE NO:'" . $linenum . "'";

		/* let's ignore some lesser issues */
		if (substr_count($errmsg, "date_default_timezone")) return;
		if (substr_count($errmsg, "Only variables")) return;

		/* log the error to the Cacti log */
#		cacti_log("PROGERR: " . $err, false, "pollperf");
		print("PROGERR: " . $err . "\n");# print_r($vars); print("</pre>");
	}

	return;
}

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
   die("<br><strong>This script is only meant to run at the command line.</strong>");
}

/* We are not talking to the browser */
$no_http_headers = true;

$dir = dirname(__FILE__);
chdir($dir);

if (strpos($dir, 'pollperf') !== false) {
	chdir('../../');
}

/* include important functions */
include_once("./include/config.php");
#include_once($config["base_path"] . "/lib/poller.php");

/* process calling arguments */
$parms = $_SERVER["argv"];
array_shift($parms);

$debug          = FALSE;
#$forcerun       = FALSE;
#$forcerun_maint = FALSE;

foreach($parms as $parameter) {
	@list($arg, $value) = @explode("=", $parameter);

	switch ($arg) {
	case "-d":
		$debug = TRUE;
		break;
	case "-h":
		display_help();
		exit;
#	case "-f":
#		$forcerun = TRUE;
#		break;
	case "-v":
	case "-V":
		display_help();
		exit;
	case "--version":
		display_help();
		exit;
	case "--help":
		display_help();
		exit;
	default:
		print "ERROR: Invalid Parameter " . $parameter . "\n\n";
		display_help();
		exit;
	}
}

/* install signal handlers for UNIX only */
if (function_exists("pcntl_signal")) {
	pcntl_signal(SIGTERM, "sig_handler");
	pcntl_signal(SIGINT, "sig_handler");
	pcntl_signal(SIGHUP, "sig_handler");
}

# let's do it, now!
cacti_log("Poller Performance Parser is now running", true, "POLLPERF");
pollperf_fill_table();
cacti_log("Poller Performance Parser has finished", true, "POLLPERF");

?>