<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2006 Reinhard Scheck aka gandalf                          |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 */

function pollperf_version () {
	return array( 'name' 	=> 'pollperf',
			'version' 	=> '0.32',
			'longname'	=> 'Poller Performance Report',
			'author'	=> 'Reinhard Scheck',
			'homepage'	=> 'http://forums.cacti.net/viewtopic.php?t=19986',
			'email'		=> '',
			'url'		=> ''
			);
}

function plugin_init_pollperf() {
	global $plugin_hooks;
	$plugin_hooks['config_arrays']['pollperf'] 			= 'pollperf_config_arrays';
	$plugin_hooks['draw_navigation_text']['pollperf'] 	= 'pollperf_draw_navigation_text';
	$plugin_hooks['config_settings']['pollperf'] 		= 'pollperf_config_settings';
	$plugin_hooks['poller_bottom']['pollperf'] 			= 'pollperf_poller_bottom';
}

function pollperf_config_arrays () {
	global $user_auth_realms, $user_auth_realm_filenames, $menu;

	$user_auth_realm_filenames['pollperf.php'] = 8;
	$temp = $menu["Utilities"]['logout.php'];
	unset($menu["Utilities"]['logout.php']);
	$menu["Utilities"]['plugins/pollperf/pollperf.php'] = "Poller Performance";
	$menu["Utilities"]['logout.php'] = $temp;
}

function pollperf_draw_navigation_text ($nav) {
	$nav["pollperf.php:"] = array("title" => "Poller Performance", "mapping" => "index.php:", "url" => "pollperf.php", "level" => "1");

	return $nav;
}

function pollperf_config_settings () {
	global $tabs, $settings, $config;

	$tabs["misc"] = "Misc";

	$temp = array(
	"plugin_pollperf_header" => array(
		"friendly_name" => "Poller Performance Settings",
		"method" => "spacer",
		),
	"plugin_pollperf_enabled" => array(
	"friendly_name" => "Enable automatic Log Verbosity Management for PollPerf",
	"description" => "PollPerf will automatically switch Log Verbosity on and off to retrieve data.",
	"method" => "checkbox",
	"default" => "off"
		),
	"plugin_pollperf_debug_start" => array(
	"friendly_name" => "Starting Time (Log Verbosity will be switched to DEBUG)",
	"description" => "Start Time for Poller DEBUG Mode",
	"method" => "textbox",
	"default" => "07:00",
	"max_length" => "5"
		),
	"plugin_pollperf_debug_count" => array(
	"friendly_name" => "Number of DEBUG Cycles",
	"description" => "The Number of DEBUG cycles. Log Verbosity will be switched to previous settings.",
	"method" => "textbox",
	"default" => "10",
	"max_length" => "3"
		)
	);
	if (isset($settings["misc"]))
	$settings["misc"] = array_merge($settings["misc"], $temp);
	else
		$settings["misc"]=$temp;
	
}

function pollperf_poller_bottom () {
	global $config;

	/*
	 * skip if function is not enabled
	 */
	$enabled = read_config_option("plugin_pollperf_enabled");
	if ($enabled == "") {
		# recall previous activation status
		$previous_status = read_config_option("plugin_pollperf_enabled2");
		if ($previous_status != "") {
			# user just switched PollPerf off
			# make sure to reset everything as if PollPerf was terminated as usual
			$orig_log_verbosity = read_config_option("plugin_pollperf_previous_verbosity");
			db_execute("update settings set value = '" . $orig_log_verbosity . "' where name = 'log_verbosity'");
			db_execute("update settings set value = '0' where name = 'plugin_pollperf_debug_cycles'");
			db_execute("replace into settings (name,value) values ('plugin_pollperf_enabled2', '" . $enabled . "')");
			cacti_log("PollPerf terminated on user request", FALSE, "POLLPERF");

			$command_string = read_config_option("path_php_binary");
			$extra_args = "-q " . $config["base_path"] . "/plugins/pollperf/pollperf_parse.php";
			exec_background($command_string, "$extra_args");
		}
		# pollperf not enabled, return
		if (read_config_option("log_verbosity", TRUE) == POLLER_VERBOSITY_DEBUG) {
			cacti_log("PollPerf not enabled", FALSE, "POLLPERF");
		}
		return;
	}

	/*
	 * pollperf is enabled
	 * set log_verbosity if start_time is reached
	 */
	$curr_time = time();
	# calculate start time as time_since_epoch for the current day
	$start = strtotime(date("Y-m-d", $curr_time) . " " . read_config_option("plugin_pollperf_debug_start"));
	/* Check for the polling interval, only valid with the Multipoller patch */
	$poller_interval = read_config_option("poller_interval");
	if (!isset($poller_interval)) {
		$poller_interval = 300;
	}

#	cacti_log("PollPerf times: " . $start . "/" . $curr_time . "/" . $poller_interval, FALSE, "POLLPERF");
	if ($start <= $curr_time &&
		$curr_time < $start + $poller_interval) {
		cacti_log("PollPerf STARTING", FALSE, "POLLPERF");
		# remember current log_verbosity
		$current_log_verbosity = read_config_option("log_verbosity");
		db_execute("replace into settings (name,value) values ('plugin_pollperf_previous_verbosity', " . $current_log_verbosity . ")");
		# set verbosity to DEBUG
		db_execute("update settings set value = '" . POLLER_VERBOSITY_DEBUG . "' where name = 'log_verbosity'");
		# set cycle count
		$count	 = read_config_option("plugin_pollperf_debug_count") + 1;
#		cacti_log("PollPerf count: " . $count, FALSE, "POLLPERF");
		db_execute("replace into settings (name,value) values ('plugin_pollperf_debug_cycles', " . $count . ")");
		# remember activation status
		db_execute("replace into settings (name,value) values ('plugin_pollperf_enabled2', '" . $enabled . "')");
	}

	/* decrease cycle count
	 * if all cycles were executed, reset log_verbosity to previous setting 
	 * */
	$cycles	= read_config_option("plugin_pollperf_debug_cycles");
	cacti_log("PollPerf cycles: " . $cycles, FALSE, "POLLPERF");
	if ($cycles > 1) {
		# decrease cycle count
		db_execute("update settings set value = '" . ($cycles - 1) . "' where name = 'plugin_pollperf_debug_cycles'");
	} elseif ($cycles == 1) {
		# reset log_verbosity and decrease cycle count
		# make sure, it is not set to a higher value than LOW
		# else there is a chance, that the cacti.log increases beyond limits
		$orig_log_verbosity = min(read_config_option("plugin_pollperf_previous_verbosity"), POLLER_VERBOSITY_LOW);
		db_execute("update settings set value = '" . $orig_log_verbosity . "' where name = 'log_verbosity'");
		db_execute("update settings set value = '" . ($cycles - 1) . "' where name = 'plugin_pollperf_debug_cycles'");
		cacti_log("PollPerf ENDING, reset verbosity to: " . $orig_log_verbosity, FALSE, "POLLPERF");

		$command_string = read_config_option("path_php_binary");
		$extra_args = "-q " . $config["base_path"] . "/plugins/pollperf/pollperf_parse.php";
		exec_background($command_string, "$extra_args");
	} elseif ($cycles == 0) {
		# nothing, for documentation purpose only
	} elseif ($cycles < 0) {
		# this should not occur
		# but to make SURE, that DEBUG level is not running forever
		# cycle count is reset to 0
		db_execute("update settings set value = '0' where name = 'plugin_pollperf_debug_cycles'");

		$command_string = read_config_option("path_php_binary");
		$extra_args = "-q " . $config["base_path"] . "/plugins/pollperf/pollperf_parse.php";
		exec_background($command_string, "$extra_args");
	}

}

?>
